# This file is part of Pantry, command-line nutrient analysis.
# 
# Version 23, released Friday, November 09, 2007.
# 
# Copyright 2007 Omari Norman.
# 
# Permission is hereby granted, free of charge, to any person
# obtaining a copy of this software and associated
# documentation files (the "Software"), to deal in the
# Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute,
# sublicense, and/or sell copies of the Software, and to
# permit persons to whom the Software is furnished to do so,
# subject to the following conditions:
# 
# The above copyright notice and this permission notice shall
# be included in all copies or substantial portions of the
# Software.
# 
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY
# KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
# WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
# PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS
# OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR
# OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR
# OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
# SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
# 
# Pantry's "built-in" XML validator is a modified version of
# xmlproc.  xmlproc is Copyright 1998-2000 by Lars Marius
# Garshol, Oslo, Norway.
# 
# All Rights Reserved
# 
# Permission to use, copy, modify, and distribute this
# software and its documentation for any purpose and without
# fee is hereby granted, provided that the above copyright
# notice appear in all copies and that modified copies are
# clearly marked as such.
# 
# LARS MARIUS GARSHOL DISCLAIMS ALL WARRANTIES WITH REGARD TO
# THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL LARS MARIUS
# GARSHOL BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL
# DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF
# USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT,
# NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
# CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.

# This module contains common functionality used by xvcmd.py and xpcmd.py

from xml.parsers.xmlproc import xmlapp, utils

# Backwards compatibility declarations

ESISDocHandler = utils.ESISDocHandler
Canonizer = utils.Canonizer
DocGenerator = utils.DocGenerator

# Error handler

class MyErrorHandler(xmlapp.ErrorHandler):

    def __init__(self, locator, parser, warnings, entstack, rawxml):
        xmlapp.ErrorHandler.__init__(self,locator)
        self.show_warnings=warnings
        self.show_entstack=entstack
        self.show_rawxml=rawxml
        self.parser=parser
        self.reset()

    def __show_location(self,prefix,msg):
        print "%s:%s: %s" % (prefix,self.get_location(),msg)
        if self.show_entstack:
            print "  Document entity"
            for item in self.parser.get_current_ent_stack():
                print "  %s: %s" % item
        if self.show_rawxml:
            raw=self.parser.get_raw_construct()
            if len(raw)>50:
                print "  Raw construct too big, suppressed."
            else:
                print "  '%s'" % raw

    def get_location(self):
        return "%s:%d:%d" % (self.locator.get_current_sysid(),\
                               self.locator.get_line(),
                               self.locator.get_column())

    def warning(self,msg):
        if self.show_warnings:
            self.__show_location("W",msg)
            self.warnings=self.warnings+1

    def error(self,msg):
        self.fatal(msg)

    def fatal(self,msg):
        self.__show_location("E",msg)
        self.errors=self.errors+1

    def reset(self):
        self.errors=0
        self.warnings=0
